import os

from bs4 import BeautifulSoup

from . import defaults as defs


def get_template_path(name):
    """
    Given a template name, it returns the relative path from the template dir.
    """
    path = os.path.join(defs.ADWARE_TEMPLATE_BASE_DIR, name)
    return path


def get_adsense_attribute(html):
    ad_client = ''
    ad_slot = ''
    if html:
        soup = BeautifulSoup(html, "html.parser")
        tag = soup.find('ins')
        ad_client = tag.get('data-ad-client', '')
        ad_slot = tag.get('data-ad-slot', '')
    return ad_client, ad_slot


def get_percentage_per_view(percentage, view_count):
    """
    Calculates new ad impression percentage based on total view count.
    """
    if not defs.ADWARE_IMPRESSION_REWARD_ENABLED:
        return percentage

    # special case when user is awarded > 80 %
    if percentage >= 80:
        return percentage

    # reward good content by number views
    new_percentage = percentage
    for item in defs.ADWARE_IMPRESSION_REWARD:
        level, reward = item
        if view_count >= level:
            new_percentage = percentage + reward
            break

    # cap max reward
    if new_percentage > defs.ADWARE_MAX_PERCENTAGE_REWARD:
        new_percentage = defs.ADWARE_MAX_PERCENTAGE_REWARD
    return new_percentage



from django.http import HttpRequest
from django.test import TestCase

from .. import utils as util
from .. import defaults as defs


class CommonTestCase(TestCase):
    """
    Common Tests.
    """
    def test_attributes(self):
        client = 'ca-pub-905269603332983480'
        slot = '3864455494'
        ad = """ <script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
            <ins class="adsbygoogle"
                 style="display:block"
                 data-ad-client="{}"
                 data-ad-slot="{}"
                 data-ad-format="auto"></ins>
            <script>
            </script>
        """
        info = util.get_adsense_attribute(ad.format(client, slot))
        self.assertEqual(info, (client, slot))

    def test_percentage_per_view_999(self):
        percentage = 30
        viewcount = 999
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(percentage, value)

    def test_percentage_per_view_4999(self):
        percentage = 30
        viewcount = 4999
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(percentage + 10, value)

    def test_percentage_per_view_9999(self):
        percentage = 30
        viewcount = 9999
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(percentage + 20, value)

    def test_percentage_per_view_14999(self):
        percentage = 30
        viewcount = 14999
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(percentage + 30, value)

    def test_percentage_per_view_19999(self):
        percentage = 30
        viewcount = 19999
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(percentage + 40, value)

    def test_percentage_per_view_29999(self):
        percentage = 30
        viewcount = 29999
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(percentage + 50, value)

    def test_percentage_per_view_reward_cap(self):
        percentage = 60
        viewcount = 50000
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(defs.ADWARE_MAX_PERCENTAGE_REWARD, value)

    def test_percentage_per_view_reward_override(self):
        percentage = 90
        viewcount = 999
        value = util.get_percentage_per_view(percentage, viewcount)
        self.assertEqual(90, value)



from django.conf import settings

ADWARE_TEMPLATE_BASE_DIR = getattr(settings, 'ADWARE_TEMPLATE_BASE_DIR', 'ads')
ADWARE_MINIMUM_IMPRESSION_PERCENTAGE = getattr(settings, 'ADWARE_MINIMUM_IMPRESSION_PERCENTAGE', 30)

ADWARE_DEFAULT_AD_CLIENT = getattr(settings, 'ADWARE_DEFAULT_AD_CLIENT', '')
ADWARE_DEFAULT_AD_SLOT = getattr(settings, 'ADWARE_DEFAULT_AD_SLOT', '')

ADWARE_IMPRESSION_REWARD_ENABLED = getattr(settings, 'ADWARE_IMPRESSION_REWARD_ENABLED', True)
ADWARE_IMPRESSION_REWARD = [[20000, 50], [15000, 40], [10000, 30], [5000, 20], [1000, 10]]  # Must be in reverse order
ADWARE_MAX_PERCENTAGE_REWARD = 80



from django.conf import settings
from django.conf.urls import url

from .views import *
from . import defaults as defs


urlpatterns = [

    url(
        r'^update$',
        AdSenseView.as_view(),
        name='adsense_update'
    ),

]



from django.contrib.auth import get_user_model
from django.http import HttpRequest
from django.test import TestCase

from ..models import AdSense
from .. import utils as util
from .. import defaults as defs

User = get_user_model()


class CommonTestCase(TestCase):
    """
    Model Tests.
    """
    site_value = (defs.ADWARE_DEFAULT_AD_CLIENT, defs.ADWARE_DEFAULT_AD_SLOT)
    user_value = ('ca-pub-905269603332983480', '3864455494')

    def setUp(self):
        user = User.objects.create(email='foo@bar.com')
        user.adsense.code = 'foobar'
        user.adsense.ad_client = 'ca-pub-905269603332983480'
        user.adsense.ad_slot='3864455494'
        user.adsense.active = True
        user.adsense.save()

    def test_user_is_active(self):
        user = User.objects.get(email='foo@bar.com')
        self.assertEqual(user.is_active, True)

    def test_adsense_is_active(self):
        user = User.objects.get(email='foo@bar.com')
        self.assertEqual(user.adsense.active, True)

    def test_adsense_get_percentage(self):
        user = User.objects.get(email='foo@bar.com')
        viewcount = 999
        percentage = user.adsense.get_percentage(viewcount)
        self.assertEqual(percentage, 30)

        viewcount = 4999
        percentage = user.adsense.get_percentage(viewcount)
        self.assertEqual(percentage, 40)

    def test_adsense_get_info_site(self):
        user = User.objects.get(email='foo@bar.com')
        user.adsense.percentage = 0
        info = user.adsense.get_info()
        self.assertEqual(info, self.site_value)

    def test_adsense_get_info_user(self):
        user = User.objects.get(email='foo@bar.com')
        user.adsense.percentage = 100
        info = user.adsense.get_info()
        self.assertEqual(info, self.user_value)

    def test_adsense_get_info_no_code(self):
        user = User.objects.get(email='foo@bar.com')
        user.adsense.percentage = 100
        user.adsense.code = ''
        info = user.adsense.get_info()
        self.assertEqual(info, self.site_value)

    def test_adsense_get_info_not_active(self):
        user = User.objects.get(email='foo@bar.com')
        user.adsense.percentage = 100
        user.adsense.active = False
        info = user.adsense.get_info()
        self.assertEqual(info, self.site_value)

    def test_adsense_get_info_suspended(self):
        user = User.objects.get(email='foo@bar.com')
        user.adsense.percentage = 100
        user.adsense.suspended = True
        info = user.adsense.get_info()
        self.assertEqual(info, self.site_value)



from django.contrib import messages
from django.views.generic import UpdateView
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponseRedirect

from toolware.utils.mixin import LoginRequiredMixin
from toolware.utils.mixin import CsrfProtectMixin
from toolware.utils.mixin import DeleteMixin

from .forms import AdSenseForm
from . import utils as util


class AdSenseView(LoginRequiredMixin, CsrfProtectMixin, UpdateView):
    """
    AdSense View
    """
    form_class = AdSenseForm
    success_url = reverse_lazy('adware:adsense_update')

    def get_template_names(self):
        template_name = util.get_template_path("adsense_form.html")
        return template_name

    def get_context_data(self, **kwargs):
        context = super(AdSenseView, self).get_context_data(**kwargs)
        context['adsense'] = self.object
        return context

    def get_object(self, queryset=None):
        ad = self.request.user.adsense
        return ad

    def form_valid(self, form):
        self.object = form.save()
        if self.object.code:
            messages.add_message(self.request, messages.SUCCESS, _('Your AdSense code was saved.'))
        return HttpResponseRedirect(self.get_success_url())



from django.conf import settings
from django.db.models import signals
from django.apps import AppConfig as DjangoAppConfig

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


class AppConfig(DjangoAppConfig):
    """
    Configuration entry point for the adware app
    """
    label = name = 'adware'
    verbose_name = "adware"

    def ready(self):
        """
        App is imported and ready, so bootstrap it.
        """
        from . import receivers as rcv

        signals.post_save.connect(rcv.create_ad, sender=AUTH_USER_MODEL)



import random

from django.db import models
from django.conf import settings
from django.utils.translation import gettext as _
from django.core.validators import MinValueValidator
from django.core.validators import MaxValueValidator
from django.utils.encoding import python_2_unicode_compatible

from . import utils as util
from . import defaults as defs

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


@python_2_unicode_compatible
class AdSense(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    user = models.OneToOneField(
        AUTH_USER_MODEL,
        related_name="%(class)s",
        editable=False,
    )

    code = models.TextField(
        _("Adsense Code"),
        blank=True,
        null=True,
        help_text=_('Your Google "Responsive" Adsense Code.'),
    )

    ad_client = models.CharField(
        max_length=60,
        blank=True,
        null=True,
    )

    ad_slot = models.CharField(
        max_length=50,
        blank=True,
        null=True,
    )

    percentage = models.PositiveIntegerField(
        default=defs.ADWARE_MINIMUM_IMPRESSION_PERCENTAGE,
        validators=[MinValueValidator(1), MaxValueValidator(100)],
        help_text=_('Ad impression percentage (1-100).'),
    )

    active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_('Active shows your ads.'),
    )

    suspended = models.BooleanField(
        _('suspended'),
        default=False,
        help_text=_('Suspended AdShare will not show ads. For admin use.'),
    )

    def __str__(self):
        return 'Adsense: {} [p:{}, a:{}]'.format(self.user.username, self.percentage, self.active)

    def get_percentage(self, view_count):
        percentage = util.get_percentage_per_view(self.percentage, view_count)
        return percentage

    def get_info(self, view_count=0):
        """
        Returns ad client/slot or ''
        """
        info = (defs.ADWARE_DEFAULT_AD_CLIENT, defs.ADWARE_DEFAULT_AD_SLOT)
        if self.user.is_active and self.active and not self.suspended:
            if self.code and self.ad_client and self.ad_slot:
                rand = random.randint(1, 100)
                percentage = self.get_percentage(view_count)
                if rand <= percentage:
                    info = (self.ad_client, self.ad_slot)
        return info



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}
SECRET_KEY = "un33k"
MIDDLEWARE_CLASSES = []

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.contenttypes',
    'adware',
)

ADWARE_IMPRESSION_REWARD_ENABLED = True
ADWARE_DEFAULT_AD_CLIENT = 'ca-pub-905269603332983333'
ADWARE_DEFAULT_AD_SLOT = '3864455333'



from django import forms
from django.utils.translation import ugettext_lazy as _

from toolware.utils.mixin import CleanSpacesMixin

from .models import AdSense
from . import utils as util


class AdSenseForm(CleanSpacesMixin, forms.ModelForm):
    """
    Custom AdSense Update Form, Simple.
    """
    def __init__(self, *args, **kwargs):
        super(AdSenseForm, self).__init__(*args, **kwargs)
        self.fields['code'].widget.attrs['autofocus'] = ''

    def clean_code(self):
        code = self.cleaned_data.get('code', '')
        if code:
            ad_client, ad_slot = util.get_adsense_attribute(code)
            if not ad_client:
                raise forms.ValidationError(_("Invalid AdSense Code. Unable to extract data-ad-client attribute."))
            if not ad_slot:
                raise forms.ValidationError(_("Invalid AdSense Code. Unable to extract data-ad-slot attribute."))
        return code

    class Meta:
        model = AdSense
        fields = ('code', 'active')

    def save(self, commit=True):
        instance = super(AdSenseForm, self).save(commit=False)
        instance.ad_client, instance.ad_slot = util.get_adsense_attribute(self.cleaned_data['code'])
        if commit:
            instance.save()
        return instance



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-25 15:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adware', '0002_auto_20160324_2006'),
    ]

    operations = [
        migrations.AlterField(
            model_name='adsense',
            name='active',
            field=models.BooleanField(default=True, help_text='Active shows your ads.', verbose_name='active'),
        ),
    ]



from django.db import transaction
from django.db import IntegrityError

from . models import AdSense


def create_ad(sender, instance, created, *args, **kwargs):
    """
    Create an ad object upon user creation.
    """
    created = False
    try:
        obj = AdSense.objects.get(user=instance)
    except AdSense.DoesNotExist:
        try:
            with transaction.atomic():
                obj = AdSense.objects.create(user=instance)
                created = True
        except IntegrityError:
            obj = AdSense.objects.get(user=instance)



from django import template
from django.conf import settings

from .. import defaults as defs

register = template.Library()


@register.assignment_tag(takes_context=True)
def adsense_info(context, user=None, view_count=0):
    """
    Returns AdSense info
    """
    if user:
        info = user.adsense.get_info(view_count)
    else:
        info = (defs.ADWARE_DEFAULT_AD_CLIENT, defs.ADWARE_DEFAULT_AD_SLOT)
    return info



from django.contrib import admin

from .models import AdSense


class AdSenseAdmin(admin.ModelAdmin):
    list_display = [
        "id",
        "user",
        "percentage",
        "ad_client",
        "ad_slot",
        "active",
        "suspended",
        "updated_at",
    ]
admin.site.register(AdSense, AdSenseAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-24 23:03
from __future__ import unicode_literals

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AdSense',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.TextField(blank=True, help_text='Your Google "Responsive" Adsense Code.', null=True, verbose_name='Adsense')),
                ('ad_client', models.CharField(blank=True, max_length=60, null=True)),
                ('ad_slot', models.CharField(blank=True, max_length=50, null=True)),
                ('percentage', models.PositiveIntegerField(default=30, help_text='Ad impression percentage (1-100).', validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(100)])),
                ('is_active', models.BooleanField(default=True, help_text='Inactive ads will not be shown.', verbose_name='active')),
                ('user', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='adsense', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-25 00:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adware', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='adsense',
            name='is_active',
        ),
        migrations.AddField(
            model_name='adsense',
            name='active',
            field=models.BooleanField(default=True, help_text='Active to show your ads.', verbose_name='active'),
        ),
        migrations.AddField(
            model_name='adsense',
            name='suspended',
            field=models.BooleanField(default=False, help_text='Suspended AdShare will not show ads. For admin use.', verbose_name='suspended'),
        ),
        migrations.AlterField(
            model_name='adsense',
            name='code',
            field=models.TextField(blank=True, help_text='Your Google "Responsive" Adsense Code.', null=True, verbose_name='Adsense Code'),
        ),
    ]
